﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "GOAPAction.h"

/**
 * @brief Checks if the preconditions are met in the world state given
 * @param WorldState A pointer to the world state to check
 * @return True if the preconditions are met and false if not
 */
bool UGOAPAction::CheckPreConditions(UWorldState* WorldState)
{
	for (TPair<FString, int> PreCondition : PreConditions)
	{
		if (WorldState->State.Contains(PreCondition.Key))
		{
			if (WorldState->State[PreCondition.Key] < PreCondition.Value) return false;
		}
	}
	return true;
}

/**
 * @brief Applies the effects of the action to the world state given
 * @param WorldState A pointer to the world state to apply the effects to
 * @return A pointer to the new world state
 */
UWorldState* UGOAPAction::ApplyEffects(UWorldState* WorldState)
{
	UWorldState* NewWorldState = NewObject<UWorldState>();
	NewWorldState->State.Append(WorldState->State);
	for (TPair<FString, int> Effect : Effects)
	{
		if (NewWorldState->State.Contains(Effect.Key))
		{
			NewWorldState->State[Effect.Key] -= Effect.Value;
		}
	}
	return NewWorldState;
}

/**
 * @brief Performs the action
 * @return A map of the effects of the action
 */
TMap<FString, int> UGOAPAction::Perform()
{
	return Effects;
}
